
/*
 * File name : genetic.c
 *
 * This program is an implementation of a Neural Network (NN) which attempts to reach a target,
 * by using a optimal search method based on this value using Genetic Algorithms (GA).
 *
 * It initially chooses several random input datasets (depending on the number of populations) 
 * and various predictions are made on these inputs. The error involved is used to "rank" the predictions.
 * The values with the lowest errors are seen as the "fittest" and are more likely to reproduce and get 
 * closer to the target. The process can stop after the prescribed number of generations, or if the target has been reached before the max. number of generations requested.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <sys/time.h>

/* NN related */
#define NUM		17	/* Total number of genes */
#define LIMIT		150	/* Maximum number of inputs the system can handle */
#define SESSIONS	1000	/* Number of generations that we'll put the system through */

/* GA related */
#define POPS		2	/* Number of populations */
#define SIZE		17	/* Size of vector in the genetic algorithms */
#define MAXPOP		20	/* Size of population */
#define BESTPOP		8	/* Number of individuals taken from the best */
#define SELPOP		12	/* SELPOP-BESTPOP = Number of people selected randomly to exchange genes within their own population */
#define NEWPOP		16	/* NEWPOP-SELPOP = Number of new people created randomly on each gen. */
#define MUT1		18	/* MUT1-NEWPOP = Number of genes that are severely mutated */
#define MIXGEN		1	/* Number of generations between population mixing */

/* result selection */
#define TARGET_UNNORMAL			35.0
#define MAX_UNNORMAL			50.0
#define MIN_UNNORMAL			14.0
#define TARGET_PERCENT_PERMIT		0.1
#define UNCERTAINTY_PERCENT_PERMIT	0.15


#define CAR		-0.03703703	/* Carbon */ 
#define MAN		0.28512388	/* Manganese */ 
#define SIL		0.03913045	/* Silicon */ 
#define PHO		0.03125002	/* Phosphorus */ 
#define SUL		-0.3222222	/* Sulphur */ 
#define CHR		-0.1		/* Chromium */ 
#define NIC		0.1		/* Nickel */ 
#define MOL		-0.33848798     /* Molybdenum */ 
#define TIT		-0.42857143	/* Titanium */ 
#define NIO		-0.48947367	/* Niobium */  
#define VAN		-0.5		/* Vanadium */ 
#define ALU		-0.5		/* Aluminium */ 
#define NIT		-0.11728396	/* Nitrogen */ 
#define BOR		-0.4333333	/* Boron */ 
#define COP		-0.01428570	/* Copper */ 
#define FRT		0.1		/* Finishin Mill Temperature */
#define CIT		0.1		/* COiling Temperature */

/* BESTPOP x POPS = no of lines in output file*/

typedef struct
{
	float p[NUM];
} vector;


/* NN related */
vector test[LIMIT], w1,target;
int hits[LIMIT], total;

/* GA related */
float pop[POPS][MAXPOP][SIZE];
double score[POPS][MAXPOP];
double result[POPS][MAXPOP];
double error[POPS][MAXPOP];
double error2[POPS][MAXPOP];

/*-----------------------------------------------------------------------*\
  |                                                                         |
  |  Randomize                                                              |
  |                                                                         |
  \*-----------------------------------------------------------------------*/
randomize()
{
	struct timeval tp;
	struct timezone tzp;

	/* Use time of day to feed the random number generator seed */
	gettimeofday( &tp, &tzp);
	srandom( tp.tv_sec );
}


/*-----------------------------------------------------------------------*\
  |                                                                         |
  |  irand( range )  - return a random integer in the range 0..(range-1)    |
  |                                                                         |
  \*-----------------------------------------------------------------------*/
int irand( range ) int range;
{
	return( random() % range );
}


/*-----------------------------------------------------------------------*\
  |                                                                         |
  |  scalar_mult - multiply two vectors                                     |
  |                                                                         |
  \*-----------------------------------------------------------------------*/
float scalar_mult( x, y ) vector x, y;
{
	int i;
	float s = 0.0;
	for ( i = 0 ; i < NUM ; i++ )  s += ( x.p[i] * y.p[i] );
	return s;
}


/*-----------------------------------------------------------------------*\
  |                                                                         |
  |  This function computes the NN's output for a certain input vector.     |
  |  The NN is constructed from 2 layers, first layer has 6 neurons,        |
  |  second layer has 1 neuron.                                             |
  |                                                                         |
  \*-----------------------------------------------------------------------*/

net(p,i) 
	int p,i;


{
	FILE *nmin,*nmin2,*nmin3,*fcount,*ferr;
	int j,k,l,m,sum,e,no_of_columns;
	char c,s[100],minmax_file[50];
	float res2,res3,res4;
	float err2,err3,err4;
	float err_a,err_b,err_c;
	float err_1,err_1a;
	float min[50],max[50],dummy,mean,stdev,dev,generation,k1,k2;
	double check_performance();
	FILE *ifp;



	printf("Calling net with p=%d and i=%d",p,i);
	if ((nmin = fopen("./input/norm_test.in","w")) == NULL ) {
		printf ("Can't write ./input/norm_test.in");
		exit(1);
	} else {

		l=0;
		for (j=0;j<SIZE;j++) {
#if 1
			fprintf(nmin,"%f ",w1.p[l]);
			++l;
#else
			if (j != 5) {
				/* change this line according to which
					variables are wanted to be fixed */
				fprintf(nmin,"%f ",w1.p[l]);
				++l;
			} else {
				if (j == 0){
					fprintf(nmin,"%f ",CAR);
				}
				if (j == 1) {
					fprintf(nmin,"%f ",MAN);
				}
				if (j == 2) {
					fprintf(nmin,"%f ",SIL);
				}
				if (j == 3) {
					fprintf(nmin,"%f ",PHO);
				}
				if (j == 4) {
					fprintf(nmin,"%f ",SUL);
				}
				if (j == 5) {
					fprintf(nmin,"%f ",CHR);
				}	 
				if (j == 6) {
					fprintf(nmin,"%f ",NIC);
				} 
				if (j == 7) {  
					fprintf(nmin,"%f ",MOL);
				}	  
				if (j == 8) {  
					fprintf(nmin,"%f ",TIT); 
				}	 
				if (j == 9) {
					fprintf(nmin,"%f ",NIO);
				}
				if (j == 10) {
					fprintf(nmin,"%f ",VAN);
				}	  
				if (j == 11) {
					fprintf(nmin,"%f ",ALU);
				}
				if (j == 12) {  
					fprintf(nmin,"%f ",NIT);
				}	   
				if (j == 13) {  
					fprintf(nmin,"%f ",BOR);
				} 
				if (j == 14) {  
					fprintf(nmin,"%f ",COP);
				}	   
				if (j == 15) {
					fprintf(nmin,"%f ",FRT);
				}	 
				if (j == 16) {
					fprintf(nmin,"%f ",CIT);
				}
			}
#endif
		}
		fprintf(nmin,"\n");
		fclose(nmin);
	}  


	sprintf(s, "./generate44 ./input/spec1.t1 8 ./input/_wh1f ./input/_wh1f.lu> /dev/null");
	system(s);
	if ( (nmin2 = fopen("_out","r")) == NULL) {
		printf ("Can't read _out stage 1");
		exit(1);
	} else {
		fscanf(nmin2,"%f %f",&res2,&err2);
		printf("res2 err2 %f %f\n",res2,err2);
		printf("Indices %d %d\n",p,i);

		fclose(nmin2);

	} 
	sprintf(s, "./generate44 ./input/spec1.t1 8 ./input/_wh4f ./input/_wh4f.lu> /dev/null");
	system(s);
	if ( (nmin2 = fopen("_out","r")) == NULL) {
		printf ("Can't read _out stage 2");
		exit(1);
	} else {
		fscanf(nmin2,"%f %f",&res3,&err3);
		printf("res3 err3 %f %f\n",res3,err3);
		fclose(nmin2);
	}

	sprintf(s, "./generate44 ./input/spec1.t1 12 ./input/_wl4f ./input/_wl4f.lu> /dev/null");
	system(s);
	if ( (nmin2 = fopen("_out","r")) == NULL) {
		printf ("Can't read _out stage 3");
		exit(1);
	} else {
		fscanf(nmin2,"%f %f",&res4,&err4);
		printf("res4 err4 %f %f\n",res4,err4);
		fclose(nmin2);
	}  

	result[p][i] = ((res2+res3+res4)/3); 
	printf("this is p %d %d\n", p,i);

	nmin3=fopen("./input/nn-input","r"); 
	fscanf(nmin3,"%f",&target.p[0]);

	err_a = pow ((res2-target.p[0]),2);
	err_b = pow ((res3-target.p[0]),2);
	err_c = pow ((res4-target.p[0]),2);

	err_1 = (err2+err3+err4);
	err_1a = pow((err_1),2);

	printf("err_a is %f\n",err_a);
	printf("err_b is %f\n",err_b);
	printf("err_c is %f\n",err_c);

	printf("err_1 is %f\n",err_1);

	error2[p][i]=sqrt(err_1a);


	error[p][i] = sqrt (((err_1a)/3)+((err_a + err_b + err_c)/3));
	printf("error pi is %f\n\n",error[p][i]);
	fcount=fopen("count","r");
	fscanf(fcount,"%d",&e); 
	fclose(fcount);


	e++;
	fcount=fopen("count","w");
	fprintf(fcount,"%d",e);


	fclose(fcount);
	printf("\n-----------> sum %d\n",e);  

	printf("In net() error is %f res is %f\n\n",error[p][i],result[p][i]);
	fclose(nmin3);

}


/*-----------------------------------------------------------------------*\
  |                                                                         |
  |  pop_swap( p, a, b )  -  swap two vectors and scores in the population p|
  |                                                                         |
  \*-----------------------------------------------------------------------*/
pop_swap( p, a, b )
	int p, a, b;

{
	int t, i;
	float tg;
	/* Swap vector */
	for ( i = 0 ; i < SIZE ; i++ ) 

	{

		t = pop[p][a][i];
		pop[p][a][i] = pop[p][b][i];
		pop[p][b][i] = t;
	}

	/* Swap score */
	tg = score[p][a];
	score[p][a] = score[p][b];
	score[p][b] = tg;

	tg = result[p][a];
	result[p][a] = result[p][b];
	result[p][b] = tg;

	tg = error[p][a];
	error[p][a] = error[p][b];
	error[p][b] = tg;


}




/*-----------------------------------------------------------------------*\
  |                                                                         |
  |  apply( p, i ) -  apply the i gene of the population p on the NN        |
  |                                                                         |
  \*-----------------------------------------------------------------------*/
apply( p, i )
	int p, i;
{
	/* Each component realtes to a gene in the NN */

	w1.p[0] = pop[p][i][0];  w1.p[1] = pop[p][i][1];  w1.p[2] = pop[p][i][2];
	w1.p[3] = pop[p][i][3];  w1.p[4] = pop[p][i][4];  w1.p[5] = pop[p][i][5];
	w1.p[6] = pop[p][i][6];  w1.p[7] = pop[p][i][7];  w1.p[8] = pop[p][i][8];
	w1.p[9] = pop[p][i][9];  w1.p[10] = pop[p][i][10]; w1.p[11] = pop[p][i][11];
	w1.p[12] = pop[p][i][12]; w1.p[13] = pop[p][i][13]; w1.p[14] = pop[p][i][14];
	w1.p[15] = pop[p][i][15]; w1.p[16] = pop[p][i][16];
}



/*-----------------------------------------------------------------------*\
  |                                                                         |
  |  pop_copy( p1, a, p2, b ) - copy the vector b in the population p2 into |
  |                             the vector a in the population p1.          |
  |                                                                         |
  \*-----------------------------------------------------------------------*/
pop_copy( p1, a, p2, b)
	int p1, a, p2, b;
{
	int i;

	for ( i = 0 ; i < SIZE ; i++ )
		pop[p1][a][i] = pop[p2][b][i];
}


/*-----------------------------------------------------------------------*\
  |                                                                         |
  |  Initialize the populations                                             |
  |                                                                         |
  \*-----------------------------------------------------------------------*/
make_initial_population()

{
	int p, i, j, k, stack = 0;
	float min[50],max[50],dummy,mean,stdev;
	char minmax_file[50],c;
	FILE *ifp;
	ifp=fopen("./input/MINMAX","r");   
	while((c=getc(ifp)) != '\n');  
	for (j = 0; j < SIZE+1; j++) {
		fscanf(ifp,"%f%f%f%f%f%f",&min[j],&max[j],&dummy,&dummy,&dummy,&dummy);
	}
	fclose(ifp);

	for ( p = 0 ; p < POPS ; p++ ) {
		/* Whole population gets values from -1 to 1 */
		for ( i = 0 ; i < MAXPOP ; i++ ) {
			for ( j = 0 ; j < SIZE ; j++ ) {
				pop[p][i][j] = ((random()&1048575) / 1000000.0 - 0.5) * 2;
				while ( pop[p][i][j] < (-(min[j]/(max[j]-min[j])+0.5))) {
					pop[p][i][j] = ((random()&1048575) / 1000000.0 - 0.5) * 2;
				}
			}
		}
	}
}




/*-----------------------------------------------------------------------*\
  |                                                                         |
  |   Calculate the scores of all the vectors in all the populations        |
  |                                                                         |
  \*-----------------------------------------------------------------------*/
calc_score()
{

	int p, i, j;
	double check_performance();

	for ( p = 0 ; p < POPS ; p++ ){
		for ( i = 0 ; i < MAXPOP ; i++ ){
			{ 
				printf("Here is scorecalc for pop %d gene %d: %f\n\n",p,i,score[p][i]);
				apply( p, i);
				score[p][i] = check_performance(p,i);
				printf("Here is scorecc2 for pop %d gene %d: %f\n\n",p,i,score[p][i]);
				printf("Read %f %f\n", result[p][i],error[p][i]);

			}
		}
	} 
}


/*-----------------------------------------------------------------------*\
  |                                                                         |
  |   Sort the populations                                                  |
  |                                                                         |
  \*-----------------------------------------------------------------------*/
sort_population()
{

	int p, i, j, k;
	float best;
	/* Use insert sort */
	for ( p = 0 ; p < POPS ; p++ ){
		for ( i = 0 ; i < MAXPOP ; i++ ){
			{
				best = score[p][i];
				for ( j = (i+1) ; j < MAXPOP ; j++ )
					if ( score[p][j] > best )
					{
						best = score[p][j];
						k = j;
					}
				if ( best > score[p][i] )
					pop_swap( p, i, k );

			}     
		}
	}

} 


/*-----------------------------------------------------------------------*\
  |                                                                         |
  |   Show (on the standard output) the best scores of all populations      |
  |                                                                         |
  \*-----------------------------------------------------------------------*/
statistics( generation )
	int generation;
{
	int p,i;
	printf("generationi is here: %d\n",generation);
	if ( generation % MIXGEN == 0 )
		printf("-----------------------------\n");
	printf(" %4d) First are: ", generation);
	for ( p = 0 ; p < POPS ; p++ )  printf("%f ", score[p][0] );
	printf(" (from %d)\n",total);
}


/*-----------------------------------------------------------------------*\
  |                                                                         |
  |  Generate the next generation in all populations                        |
  |                                                                         |
  \*-----------------------------------------------------------------------*/
make_next_generation( generation )
	int generation;
{
	int a, p, i, j, k, k1, k2, m, stack;
	float dev,min[50],max[50],dummy,mean,stdev;
	char minmax_file[50],c;
	FILE *ifp;
	ifp=fopen("./input/MINMAX","r");
	while((c=getc(ifp)) != '\n');
	for (j = 0; j < SIZE+1; j++) {
		fscanf(ifp,"%f%f%f%f%f%f",&min[j],&max[j],&dummy,&dummy,&dummy,&dummy);
	}
	fclose(ifp);


	for ( p = 0 ; p < POPS ; p++ ) {
		/* keep best - BESTPOP */
		/* add another group, randomly  - (SELPOP-BESTPOP) */
		for ( i = BESTPOP ; i < SELPOP ; i++ ) {
			pop_swap( p, i, (irand( MAXPOP - i ) + i) );
		}      
		/* create new individuals */
		for ( i = MUT1 ; i < MAXPOP ; i++ ) {

			stack = 0;
			for ( j = 0 ; j < SIZE ; j++ ) {
				pop[p][i][j] = ((random()&1048575) / 1000000.0 - 0.5) * 2;
				while ( pop[p][i][j] < (-(min[j]/(max[j]-min[j])+0.5))) {
					pop[p][i][j] = ((random()&1048575) / 1000000.0 - 0.5) * 2;
				}
			}
		}      
		/*  SELPOP to MUT1 will be severe mutations */

		stack = 0;
		for ( i = NEWPOP ; i < MUT1 ; i++ ) {
			pop_copy( p, i, p, irand(NEWPOP) );
			/* 5000 is the nominal mutation value */ 
			dev = 1 + ((irand(2000) - 1000 )/ 5000);
			j=irand(SIZE);

			while ( dev < (-(min[j]/(max[j]-min[j])+0.5))) {
				dev = 1 + ((irand(2000) - 1000 )/ 5000);
			}
			pop[p][i][j] = dev;
		} 

		/* MUT2 to MAXPOP will be crossovers */

		stack = 0;
		for ( i = SELPOP ; i < NEWPOP ; i++ ) {

			/* Every several generations (set by MIXGEN) there is a cross-over */
			/*   between different populations. */
			pop_copy( p, i, (((generation%MIXGEN)==0) ? irand(POPS) : p), irand(NEWPOP) );
			j = irand(NEWPOP);
			k1 = irand( SIZE - 1);
			k2 = irand( SIZE - 1 - k1 ) + k1 + 1;
			for ( m = k1 ; m <= k2 ; m++ ) pop[p][i][m] = pop[p][j][m];

			/* Mutate slightly to preserve diversity */
			dev = 1 + ((irand(2000) - 1000 )/ 50000);
			j=irand(SIZE);
			while ( dev < (-(min[j]/(max[j]-min[j])+0.5))) {
				dev = 1 + ((irand(2000) - 1000 )/ 50000);
			}
			pop[p][i][j] = dev;
		}
	}
	calc_score();
	sort_population();
	statistics( generation );
	printf("Done %d generations\n\n",generation);

}




/*-----------------------------------------------------------------------*\
  |                                                                         |
  |  Return the number of cases for which the NN returns the correct value  |
  |                                                                         |
  \*-----------------------------------------------------------------------*/
double check_performance (p,i) int p,i;
{
	vector x;
	int j;
	float score_value=0,score_a,d,f;
	net(p,i);


	printf("score_a is %f\n",score_a); 

	score_value = error[p][i];

	printf ("Answer is %f and target is %f\n",result[p][i],target.p[1]);
	if (score_value < 0) {
		score_value = 0; }
	else { 
		score_value = 1.0/(score_value);
	}  
	printf("score_value is: %f\n",score_value);
	return score_value;
}


/*-----------------------------------------------------------------------*\
  |                                                                         |
  |  Return the number of cases for which the NN returns the correct value  |
  |                                                                         |
  \*-----------------------------------------------------------------------*/
calc_limit_err()
{
	vector x;
	int j;
	float err=0;

	err = pow((target.p[2]*target.p[0]),2);
	target.p[1] = 1.0/sqrt(err) ;
	printf ("Norm Target Value is %f\n",target.p[0]);
	printf("Target Score is: %f\n",target.p[1]);
	return err;
}


/*-----------------------------------------------------------------------*\
  |                                                                         |
  |  Get data (reads input file)                                            |
  |                                                                         |
  \*-----------------------------------------------------------------------*/
int get_data()
{
	char* FileName = "./input/nn-input";
	FILE *fd;
	int i, posnum, negnum;
	float x,y,t;

	/* opens the file  */
	if ( (fd = fopen(FileName,"r")) == NULL )
	{
		printf ("no-input-file");
		exit(10);
	}

	/* Total number of input values */
	total = 0;

	fscanf(fd,"%f",&t);
	target.p[0]=t;
	printf("Target is %f",target.p[0]);

	fscanf(fd,"%f",&t);
	target.p[2]=t;
	printf("Wanted accuracy is %f",target.p[2]);

	fclose( fd );
	return (0) ;
}


/*-----------------------------------------------------------------------*\
  |                                                                         |
  |   best_pop  -  Find the population with the best solution               |
  |                                                                         |
  \*-----------------------------------------------------------------------*/
int best_pop()
{
	int i, p;
	float best=0;
	for ( i = 0 ; i < POPS ; i++ )
		if ( score[i][0] > best )
		{
			best = score[i][0];
			p = i;
		}
	return(p);
}



/*-----------------------------------------------------------------------*\
  |                                                                         |
  |  Main                                                                   |
  |                                                                         |
  \*-----------------------------------------------------------------------*/



main() {
	int generation, i, j, l, p, best, done = 0,e=0, err_1a, k,z;
	float px, py, px1, py1, p0,p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,er1,er2;
	FILE *fp,*fb,*fcount, *ferr,*fscore;
	FILE *fd;
	double tempResult, tempUncertainty;

	fcount=fopen("count","w");
	fprintf(fcount,"%d",e);
	fclose(fcount);



	randomize();
	get_data();  /* Read input from file */
	calc_limit_err();
	printf("Target is :%f",target.p[0]);
	make_initial_population();
	calc_score();
	sort_population();

	for (p=0;p<POPS;p++) {
		printf("Score of %d is %f",p,score[p][0]);
	}
	printf("We got here ");
	/* Educate the net */
	generation = 0;

	while ( (done != 1 ) && ( generation++ < SESSIONS ) )

	{

		make_next_generation( generation );
		p = best_pop();      


		if ( score[p][0] > target.p[1] ) {
			printf ("Target error was %f and error is %f",target.p[1],score[p][0]);
			done = 1; 
		} else {
			printf("Done all !!\n");

		}

		fb=fopen("./unnormalise/nn-output_b","a+");
		for (i=0;i<POPS;i++) {
			for ( j=0;j<BESTPOP;j++) {
				apply (i,j);
				l=0;
				tempResult = (result[i][j] + 0.5) * (MAX_UNNORMAL - MIN_UNNORMAL) + MIN_UNNORMAL;
				tempUncertainty = ((error[i][j] + result[i][j] + 0.5) * (MAX_UNNORMAL - MIN_UNNORMAL) + MIN_UNNORMAL) - tempResult;

				if ((fabs(TARGET_UNNORMAL - tempResult) < (TARGET_PERCENT_PERMIT * TARGET_UNNORMAL)) 
					&& (tempUncertainty < (2 * UNCERTAINTY_PERCENT_PERMIT * TARGET_UNNORMAL)))
				{
				for (k=0;k<SIZE;k++) {
#if 1
					fprintf(fb,"%f ",w1.p[l]);
					++l;
#else
					if (k != 5) {  /* change this line according to which variables are wanted to be fixed */
						fprintf(nmin,"%f ",w1.p[l]);
						++l;
					} else {
						if (k == 0){
							fprintf(nmin,"%f ",CAR);
						}
						if (k == 1) {
							fprintf(nmin,"%f ",MAN);
						}
						if (k == 2) {
							fprintf(nmin,"%f ",SIL);
						}
						if (k == 3) {
							fprintf(nmin,"%f ",PHO);
						}
						if (k == 4) {
							fprintf(nmin,"%f ",SUL);
						}
						if (k == 5) {
							fprintf(nmin,"%f ",CHR);
						}	 
						if (k == 6) {
							fprintf(nmin,"%f ",NIC);
						} 
						if (k == 7) {  
							fprintf(nmin,"%f ",MOL);
						}	  
						if (k == 8) {  
							fprintf(nmin,"%f ",TIT); 
						}	 
						if (k == 9) {
							fprintf(nmin,"%f ",NIO);
						}
						if (k == 10) {
							fprintf(nmin,"%f ",VAN);
						}	  
						if (k == 11) {
							fprintf(nmin,"%f ",ALU);
						}
						if (k == 12) {  
							fprintf(nmin,"%f ",NIT);
						}	   
						if (k == 13) {  
							fprintf(nmin,"%f ",BOR);
						} 
						if (k == 14) {  
							fprintf(nmin,"%f ",COP);
						}	   
						if (k == 15) {
							fprintf(nmin,"%f ",FRT);
						}	 
						if (k == 16) {
							fprintf(nmin,"%f ",CIT);
						}
					}
#endif
				}
				fprintf(fb,"%f %f  \n",result[i][j],(error[i][j]+result[i][j]));
				}

			}
		}

		fclose(fb);
	}
	printf("done! \n");
	printf("\n\n****  For unnormalisation, goto /unnormalise and type a.out (no. of lines in output file) => BESTPOP x POPS = no of lines in output file  (no. of inputs)****\n");
	fscore=fopen("score_output","w"); 
	for (i=0;i<POPS;i++) { 
		for ( j=0;j<MAXPOP;j++) { 
			fprintf(fscore,"Score for score_T_298 pop %d gene %d is %f \n",i,j,score[i][j]); 
			printf("scores for score_T_298 pop %d chromo %d id %f \n",i,j,score[i][j]);
		} 
	}       

	printf("\n Scores printed to 'scores'\n\n");
	fclose(fscore);
}



